@@ -1,4 +1,4 @@
-/*      $Id: xfwm4_plugin.c,v 1.67 2004/08/22 22:18:50 olivier Exp $
+/*      $Id: xfwm4_plugin.c,v 1.76 2004/09/26 09:02:56 olivier Exp $
  
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -31,6 +31,7 @@
 #include <string.h>
 #include <stdlib.h>
 #include <stdio.h>
+#include <fcntl.h>
 
 #include <gtk/gtk.h>
 #include <gdk/gdk.h>
@@ -48,13 +49,6 @@
 
 #define INDICATOR_SIZE 11
 
-typedef enum
-{
-    DECORATION_THEMES = 0,
-    KEYBINDING_THEMES = 1
-}
-ThemeType;
-
 typedef struct _TitleRadioButton TitleRadioButton;
 struct _TitleRadioButton
 {
@@ -103,20 +97,12 @@ MenuTmpl title_align_values[] = {
     {NULL, NULL}
 };
 
-enum
-{
-    THEME_NAME_COLUMN,
-    N_COLUMNS
-};
-
-static void create_channel (McsPlugin * mcs_plugin);
-static gboolean write_options (McsPlugin * mcs_plugin);
+static void xfwm4_create_channel (McsPlugin * mcs_plugin);
 static void run_dialog (McsPlugin * mcs_plugin);
 
 static gboolean setting_model = FALSE;
 static gboolean is_running = FALSE;
 static gchar *current_theme = NULL;
-gchar *current_key_theme = NULL;
 static gchar *current_layout = NULL;
 static gchar *current_font = NULL;
 static gchar *dbl_click_action = NULL;
@@ -134,10 +120,13 @@ static gboolean box_resize = FALSE;
 static int raise_delay;
 static int snap_width;
 static int wrap_resistance;
+gchar *current_key_theme = NULL;
 
 static GList *decoration_theme_list = NULL;
 GList *keybinding_theme_list = NULL;
 
+
+
 static void
 sensitive_cb (GtkWidget * widget, gpointer user_data)
 {
@@ -493,15 +482,21 @@ hidden_data_receive (GtkWidget * widget, GdkDragContext * drag_context,
 {
     GtkWidget *source = gtk_drag_get_source_widget (drag_context);
     GtkWidget *parent = gtk_widget_get_parent (source);
-    McsPlugin *mcs_plugin = (McsPlugin *) g_object_get_data (G_OBJECT(user_data), "mcs");
+    McsPlugin *mcs_plugin = NULL;
     
     g_return_if_fail (GTK_IS_WIDGET (user_data));
 
+    mcs_plugin = (McsPlugin *) g_object_get_data (G_OBJECT(user_data), "mcs");
     gtk_widget_ref (source);
     gtk_container_remove (GTK_CONTAINER (parent), source);
     gtk_box_pack_start (GTK_BOX (user_data), source, FALSE, FALSE, 0);
     gtk_widget_unref (source);
 
+    if (parent == GTK_WIDGET (user_data))
+    {
+        return;
+    }
+
     if (current_layout)
     {
         g_free (current_layout);
@@ -520,10 +515,11 @@ layout_data_receive (GtkWidget * widget, GdkDragContext * drag_context,
 {
     GtkWidget *source = gtk_drag_get_source_widget (drag_context);
     GtkWidget *parent = gtk_widget_get_parent (source);
-    McsPlugin *mcs_plugin = (McsPlugin *) g_object_get_data (G_OBJECT(user_data), "mcs");
+    McsPlugin *mcs_plugin = NULL;
 
     g_return_if_fail (GTK_IS_WIDGET (user_data));
 
+    mcs_plugin = (McsPlugin *) g_object_get_data (G_OBJECT(user_data), "mcs");
 
     gtk_widget_set_app_paintable (GTK_WIDGET(user_data), FALSE);
     gtk_widget_ref (source);
@@ -678,7 +674,7 @@ create_option_menu_box (MenuTmpl template[], guint size, gchar * display_label,
     return (vbox);
 }
 
-static void
+void
 theme_info_free (ThemeInfo * info)
 {
     g_free (info->path);
@@ -754,17 +750,19 @@ update_theme_dir (const gchar * theme_dir, GList * theme_list)
     gboolean set_layout = FALSE;
     gboolean set_align = FALSE;
     gboolean set_font = FALSE;
+    gboolean user_writable = FALSE;
 
     gchar *tmp;
 
-    tmp = g_build_filename (theme_dir, G_DIR_SEPARATOR_S, KEY_SUFFIX, G_DIR_SEPARATOR_S, KEYTHEMERC, NULL);
+    tmp = g_build_filename (theme_dir, KEY_SUFFIX, KEYTHEMERC, NULL);
     if (g_file_test (tmp, G_FILE_TEST_IS_REGULAR) && parserc (tmp, &set_layout, &set_align, &set_font))
     {
         has_keybinding = TRUE;
+        user_writable = (access (tmp, W_OK) == 0);
     }
     g_free (tmp);
 
-    tmp = g_build_filename (theme_dir, G_DIR_SEPARATOR_S, SUFFIX, G_DIR_SEPARATOR_S, THEMERC, NULL);
+    tmp = g_build_filename (theme_dir, SUFFIX, THEMERC, NULL);
     if (g_file_test (tmp, G_FILE_TEST_IS_REGULAR) && parserc (tmp, &set_layout, &set_align, &set_font))
     {
         has_decoration = TRUE;
@@ -789,6 +787,7 @@ update_theme_dir (const gchar * theme_dir, GList * theme_list)
             info->set_layout = set_layout;
             info->set_align = set_align;
             info->set_font = set_font;
+	    info->user_writable = user_writable;
         }
     }
     else
@@ -803,7 +802,7 @@ update_theme_dir (const gchar * theme_dir, GList * theme_list)
             info->set_layout = set_layout;
             info->set_align = set_align;
             info->set_font = set_font;
-
+	    info->user_writable = user_writable;
             list = g_list_prepend (list, info);
         }
     }
@@ -845,16 +844,18 @@ themes_common_list_add_dir (const char *dirname, GList * theme_list)
 static GList *
 theme_common_init (GList * theme_list)
 {
-    gchar *dir;
+    gchar **dirs, **d;
     GList *list = theme_list;
 
-    dir = xfce_get_homefile (".themes", NULL);
-    list = themes_common_list_add_dir (dir, list);
-    g_free (dir);
+    xfce_resource_push_path (XFCE_RESOURCE_THEMES,
+                             DATADIR G_DIR_SEPARATOR_S "themes");
+    dirs = xfce_resource_dirs (XFCE_RESOURCE_THEMES);
+    xfce_resource_pop_path (XFCE_RESOURCE_THEMES);
 
-    dir = g_build_filename (DATADIR, "themes", NULL);
-    list = themes_common_list_add_dir (dir, list);
-    g_free (dir);
+    for (d = dirs; *d != NULL; ++d)
+        list = themes_common_list_add_dir (*d, list);
+    
+    g_strfreev (dirs);
 
     return list;
 }
@@ -952,16 +953,27 @@ keybinding_selection_changed (GtkTreeSelection * selection, gpointer data)
     {
         if (current_key_theme && strcmp (current_key_theme, new_key_theme))
         {
+	    ThemeInfo *ti;
+
             g_free (current_key_theme);
             current_key_theme = new_key_theme;
             mcs_manager_set_string (mcs_plugin->manager, "Xfwm/KeyThemeName", CHANNEL, current_key_theme);
             mcs_manager_notify (mcs_plugin->manager, CHANNEL);
             write_options (mcs_plugin);
-        }
+
+	    ti = find_theme_info_by_name (new_key_theme, keybinding_theme_list);
+	    
+	    if (ti)
+	    {
+	        gtk_widget_set_sensitive (itf->treeview3, ti->user_writable);
+		gtk_widget_set_sensitive (itf->treeview4, ti->user_writable);
+		loadtheme_in_treeview (ti, itf);
+	    }
+	}
     }
 }
 
-static GList *
+GList *
 read_themes (GList * theme_list, GtkWidget * treeview, GtkWidget * swindow, ThemeType type, gchar * current_value)
 {
     GList *list;
@@ -1338,32 +1350,6 @@ cb_dialog_response (GtkWidget * dialog, gint response_id)
     }
 }
 
-static void
-cb_shortcuttheme_changed (GtkTreeSelection * selection, Itf * itf)
-{
-    ThemeInfo *ti;
-    GtkTreeModel *model;
-    GtkTreeIter iter;
-    gchar *theme_name;
-
-    if (gtk_tree_selection_get_selected (selection, &model, &iter))
-    {
-        gtk_tree_model_get (model, &iter, THEME_NAME_COLUMN, &theme_name, -1);
-
-        ti = find_theme_info_by_name (theme_name, keybinding_theme_list);
-
-        if (ti)
-        {
-            gchar *theme_file = g_build_filename (ti->path, G_DIR_SEPARATOR_S, KEY_SUFFIX, G_DIR_SEPARATOR_S, KEYTHEMERC, NULL);
-            loadtheme_in_treeview (theme_file, itf);
-
-            g_free (theme_file);
-        }
-
-        g_free (theme_name);
-    }
-}
-
 Itf *
 create_dialog (McsPlugin * mcs_plugin)
 {
@@ -1502,7 +1488,6 @@ create_dialog (McsPlugin * mcs_plugin)
     dialog->treeview3 = gtk_tree_view_new_with_model (GTK_TREE_MODEL (model));
     gtk_widget_show (dialog->treeview3);
     gtk_container_add (GTK_CONTAINER (dialog->scrolledwindow3), dialog->treeview3);
-    /* gtk_widget_set_size_request (dialog->treeview3, 250, -1); */
 
     /* command column */
     renderer = gtk_cell_renderer_text_new ();
@@ -1543,6 +1528,13 @@ create_dialog (McsPlugin * mcs_plugin)
 
     gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (dialog->treeview4), -1, _("Shortcut"), renderer, "text", COLUMN_SHORTCUT, NULL);
     gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (dialog->treeview4), TRUE);
+    /* popup menu */
+    dialog->popup_menu = gtk_menu_new ();
+    dialog->popup_add_menuitem = gtk_image_menu_item_new_from_stock (GTK_STOCK_ADD,NULL);
+    gtk_container_add (GTK_CONTAINER (dialog->popup_menu), dialog->popup_add_menuitem);
+    dialog->popup_del_menuitem = gtk_image_menu_item_new_from_stock (GTK_STOCK_REMOVE, NULL);
+    gtk_container_add (GTK_CONTAINER (dialog->popup_menu), dialog->popup_del_menuitem);
+    gtk_widget_show_all (dialog->popup_menu);
 
     label = gtk_label_new (_("Keyboard"));
     gtk_widget_show (label);
@@ -1827,10 +1819,13 @@ setup_dialog (Itf * itf)
 
     selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (itf->treeview2));
     gtk_tree_selection_set_mode (selection, GTK_SELECTION_BROWSE);
+
     g_signal_connect (G_OBJECT (selection), "changed", (GCallback) keybinding_selection_changed, itf);
 
+    g_signal_connect (G_OBJECT (itf->treeview2), "button-press-event", G_CALLBACK (cb_popup_menu), itf);
+    g_signal_connect (G_OBJECT (itf->popup_add_menuitem), "activate", G_CALLBACK (cb_popup_add_menu), itf);
+    g_signal_connect (G_OBJECT (itf->popup_del_menuitem), "activate", G_CALLBACK (cb_popup_del_menu), itf);
 
-    g_signal_connect (G_OBJECT (gtk_tree_view_get_selection (GTK_TREE_VIEW (itf->treeview2))), "changed", G_CALLBACK (cb_shortcuttheme_changed), itf);
     g_signal_connect (G_OBJECT (itf->treeview3), "row-activated", G_CALLBACK (cb_activate_treeview3), itf);
     g_signal_connect (G_OBJECT (itf->treeview4), "row-activated", G_CALLBACK (cb_activate_treeview4), itf);
 
@@ -1844,14 +1839,14 @@ setup_dialog (Itf * itf)
 
     if (ti)
     {
-        gchar *theme_file = g_build_filename (ti->path, G_DIR_SEPARATOR_S, KEY_SUFFIX, G_DIR_SEPARATOR_S, KEYTHEMERC, NULL);
-        loadtheme_in_treeview (theme_file, itf);
-
-        g_free (theme_file);
+        gtk_widget_set_sensitive (itf->treeview3, ti->user_writable);
+        gtk_widget_set_sensitive (itf->treeview4, ti->user_writable);
+        loadtheme_in_treeview (ti, itf);
     }
     else
+    {
         g_warning ("Cannot find the keytheme !");
-
+    }
     g_signal_connect (G_OBJECT (itf->xfwm4_dialog), "response", G_CALLBACK (cb_dialog_response), itf->mcs_plugin);
     g_signal_connect (G_OBJECT (itf->font_button), "clicked", G_CALLBACK (show_font_selection), itf);
     g_signal_connect (G_OBJECT (itf->click_focus_radio), "toggled", G_CALLBACK (cb_click_to_focus_changed), itf);
@@ -1877,7 +1872,7 @@ mcs_plugin_init (McsPlugin * mcs_plugin)
 {
     xfce_textdomain (GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR, "UTF-8");
 
-    create_channel (mcs_plugin);
+    xfwm4_create_channel (mcs_plugin);
     mcs_plugin->plugin_name = g_strdup (PLUGIN_NAME);
     mcs_plugin->caption = g_strdup (_("Window Manager"));
     mcs_plugin->run_dialog = run_dialog;
@@ -1888,14 +1883,26 @@ mcs_plugin_init (McsPlugin * mcs_plugin)
 }
 
 static void
-create_channel (McsPlugin * mcs_plugin)
+xfwm4_create_channel (McsPlugin * mcs_plugin)
 {
     McsSetting *setting;
+    gchar *rcfile, *path;
 
-    gchar *rcfile;
+    path = g_build_filename ("xfce4", RCDIR, RCFILE, NULL);
+    rcfile = xfce_resource_lookup (XFCE_RESOURCE_CONFIG, path);
+    g_free (path);
 
-    rcfile = xfce_get_userfile (RCDIR, G_DIR_SEPARATOR_S, RCFILE, NULL);
-    mcs_manager_add_channel_from_file (mcs_plugin->manager, CHANNEL, rcfile);
+    if (!rcfile)
+        rcfile = xfce_get_userfile (OLDRCDIR, RCFILE, NULL);
+
+    if (g_file_test (rcfile, G_FILE_TEST_EXISTS))
+    {
+        mcs_manager_add_channel_from_file (mcs_plugin->manager, CHANNEL, rcfile);
+    }
+    else
+    {
+        mcs_manager_add_channel (mcs_plugin->manager, CHANNEL);
+    }
     g_free (rcfile);
 
     setting = mcs_manager_setting_lookup (mcs_plugin->manager, "Xfwm/KeyThemeName", CHANNEL);
@@ -2160,16 +2167,21 @@ create_channel (McsPlugin * mcs_plugin)
         dbl_click_action = g_strdup (DEFAULT_ACTION);
         mcs_manager_set_string (mcs_plugin->manager, "Xfwm/DblClickAction", CHANNEL, dbl_click_action);
     }
+
+    write_options (mcs_plugin);
 }
 
-static gboolean
+gboolean
 write_options (McsPlugin * mcs_plugin)
 {
-    gchar *rcfile;
+    gchar *rcfile, *path;
     gboolean result;
 
-    rcfile = xfce_get_userfile (RCDIR, RCFILE, NULL);
+    path = g_build_filename ("xfce4", "mcs_settings", RCFILE, NULL);
+    rcfile = xfce_resource_save_location (XFCE_RESOURCE_CONFIG, path, TRUE);
+    
     result = mcs_manager_save_channel_to_file (mcs_plugin->manager, CHANNEL, rcfile);
+    g_free (path);
     g_free (rcfile);
 
     return result;
@@ -2185,6 +2197,8 @@ run_dialog (McsPlugin * mcs_plugin)
 
     is_running = TRUE;
 
+    xfce_textdomain(GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR, "UTF-8");
+
     dialog = create_dialog (mcs_plugin);
     setup_dialog (dialog);
 }
