@@ -1,4 +1,4 @@
-/*      $Id: xfwm4_plugin.h,v 1.6 2004/09/05 20:57:45 olivier Exp $
+/*      $Id: xfwm4_plugin.h,v 1.9 2004/09/25 15:32:01 pollux Exp $
  
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -21,9 +21,10 @@
 #ifndef __XFWM4_PLUGIN_H
 #define __XFWM4_PLUGIN_H
 
-#define RCDIR   "settings"
-#define RCFILE  "xfwm4.xml"
-#define CHANNEL "xfwm4"
+#define RCDIR    "mcs_settings"
+#define OLDRCDIR "settings"
+#define RCFILE   "xfwm4.xml"
+#define CHANNEL  "xfwm4"
 #define PLUGIN_NAME "xfwm4"
 
 #define DEFAULT_THEME "Default"
@@ -110,6 +111,16 @@ struct _Itf
     GtkWidget *wrap_windows_check;
     GtkWidget *wrap_resistance_scale;
     GtkWidget *xfwm4_dialog;
+    GtkWidget *popup_menu;
+    GtkWidget* popup_add_menuitem;
+    GtkWidget* popup_del_menuitem;
+
+};
+
+enum
+{
+    THEME_NAME_COLUMN,
+    N_COLUMNS
 };
 
 enum
@@ -119,9 +130,18 @@ enum
     NUM_COLUMNS
 };
 
+typedef enum
+{
+    DECORATION_THEMES = 0,
+    KEYBINDING_THEMES = 1
+}
+ThemeType;
+
 extern gchar *current_key_theme;
 extern GList *keybinding_theme_list;
 
+extern gboolean write_options (McsPlugin *);
+extern void theme_info_free (ThemeInfo *);
 extern ThemeInfo *find_theme_info_by_name (const gchar *, GList *);
-
+extern GList *read_themes (GList *, GtkWidget *, GtkWidget *, ThemeType, gchar *);
 #endif
