@@ -1,4 +1,4 @@
-/*      $Id: hints.c,v 1.85 2004/08/28 23:10:17 olivier Exp $
+/*      $Id: hints.c,v 1.88 2004/09/25 21:49:33 olivier Exp $
  
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -70,6 +70,7 @@ Atom net_current_desktop;
 Atom net_desktop_geometry;
 Atom net_desktop_viewport;
 Atom net_desktop_names;
+Atom net_desktop_layout;
 Atom net_number_of_desktops;
 Atom net_showing_desktop;
 Atom net_startup_id;
@@ -115,6 +116,7 @@ Atom net_wm_window_type_splash;
 Atom net_wm_window_type_toolbar;
 Atom net_wm_window_type_utility;
 Atom net_wm_user_time;
+Atom net_wm_opacity;
 Atom net_workarea;
 Atom utf8_string;
 
@@ -127,6 +129,11 @@ Atom kwm_win_icon;
 Atom net_system_tray_manager;
 Atom net_system_tray_opcode;
 
+/* Misc atoms */
+Atom misc_xrootpmap;
+Atom misc_xsetroot;
+Atom misc_pixmap;
+
 static gboolean
 check_type_and_format (Display * dpy, Window w, Atom a, int expected_format,
     Atom expected_type, int n_items, int format, Atom type)
@@ -213,6 +220,16 @@ initMotifHints (Display * dpy)
     motif_wm_hints = XInternAtom (dpy, "_MOTIF_WM_HINTS", FALSE);
 }
 
+void
+initMiscHints (Display * dpy)
+{
+    TRACE ("entering initMiscHints");
+
+    misc_xrootpmap = XInternAtom (dpy, "_XROOTPMAP_ID", FALSE);
+    misc_xsetroot = XInternAtom (dpy, "_XSETROOT_ID", FALSE);
+    misc_pixmap = XInternAtom (dpy, "PIXMAP", FALSE);
+}
+
 PropMwmHints *
 getMotifHints (Display * dpy, Window w)
 {
@@ -395,6 +412,75 @@ setHint (Display * dpy, Window w, Atom a, long value)
         (unsigned char *) &value, 1);
 }
 
+void
+getDesktopLayout (Display * dpy, Window root, int ws_count, NetWmDesktopLayout * layout)
+{
+    Atom real_type;
+    int real_format;
+    gboolean success = FALSE;
+    unsigned long items_read, items_left;
+    unsigned long orientation, cols, rows, start;
+    unsigned long *data = NULL;
+
+    TRACE ("entering getDesktopLayout");
+
+    if ((XGetWindowProperty (dpy, root, net_desktop_layout,
+                0L, 4L, FALSE, XA_CARDINAL,
+                &real_type, &real_format, &items_read, &items_left,
+                (unsigned char **) &data) == Success) && (items_read >= 3))
+    {
+        do
+        {
+            orientation = data[0];
+            cols = data[1];
+            rows = data[2];
+            start = (items_read >= 4) ? data[3] : NET_WM_TOPLEFT;
+
+            if (orientation > NET_WM_ORIENTATION_VERT) 
+            {
+                break;
+            }
+            
+            if (start > NET_WM_BOTTOMLEFT)
+            {
+                break;
+            }
+            
+            if ((rows == 0) && (cols == 0))
+            {
+                break;
+            }
+
+            if (rows == 0)
+            {
+                rows = (ws_count-1) / cols + 1;
+            }
+
+            if (cols == 0)
+            {
+                cols = (ws_count-1) / rows + 1;
+            }
+
+            layout->orientation = orientation;
+            layout->cols = cols;
+            layout->rows = rows;
+            layout->start = start;
+            success = TRUE;
+        } while (0);
+
+        XFree (data);
+    }
+
+    if (!success)
+    {
+        /* Assume HORZ, TOPLEFT, one row by default */
+        layout->orientation = NET_WM_ORIENTATION_HORZ;
+        layout->cols = ws_count;
+        layout->rows = 1;
+        layout->start = NET_WM_TOPLEFT;
+    }
+}
+
 void
 getGnomeDesktopMargins (Display * dpy, int screen, int * m)
 {
@@ -458,6 +544,8 @@ initNetHints (Display * dpy)
         XInternAtom (dpy, "_NET_DESKTOP_VIEWPORT", FALSE);
     net_desktop_names = 
         XInternAtom (dpy, "_NET_DESKTOP_NAMES", FALSE);
+    net_desktop_layout =
+        XInternAtom (dpy, "_NET_DESKTOP_LAYOUT", FALSE);
     net_number_of_desktops =
         XInternAtom (dpy, "_NET_NUMBER_OF_DESKTOPS", FALSE);
     net_showing_desktop = 
@@ -550,6 +638,8 @@ initNetHints (Display * dpy)
         XInternAtom (dpy, "_NET_WORKAREA", FALSE);
     net_wm_user_time = 
         XInternAtom (dpy, "_NET_WM_USER_TIME", FALSE);
+    net_wm_opacity = 
+        XInternAtom (dpy, "_NET_WM_WINDOW_OPACITY", FALSE);
     utf8_string = 
         XInternAtom (dpy, "UTF8_STRING", FALSE);
 }
@@ -569,6 +659,7 @@ setNetSupportedHint (Display * dpy, int screen, Window check_win)
     atoms[i++] = net_desktop_geometry;
     atoms[i++] = net_desktop_viewport;
     atoms[i++] = net_desktop_names;
+    atoms[i++] = net_desktop_layout;
     atoms[i++] = net_number_of_desktops;
     atoms[i++] = net_showing_desktop;
     atoms[i++] = net_supported;
