@@ -1,4 +1,4 @@
-/*      $Id: screen.c,v 1.4 2004/08/22 22:18:50 olivier Exp $
+/*      $Id: screen.c,v 1.7 2004/09/25 21:49:33 olivier Exp $
  
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -113,15 +113,9 @@ myScreenInit (DisplayInfo *display_info, GdkScreen *gscr, unsigned long event_ma
     screen_info->client_serial = 0L;
     screen_info->button_handler_id = 0L;
     
-#ifdef HAVE_COMPOSITOR
-    screen_info->root_picture = None;
-    screen_info->damage_region = None;
-    screen_info->trans_pixmap = None;
-    screen_info->trans_picture = None;
-#endif /* HAVE_COMPOSITOR */
-
     /* Create the side windows to detect edge movement */
-    xfwmWindowTemp (display_info->dpy, screen_info->xroot, &screen_info->sidewalk[0], 0, 0, 
+    xfwmWindowTemp (display_info->dpy, screen_info->xroot, &screen_info->sidewalk[0], 
+                                  0, 0, 
                                   1, gdk_screen_get_height (gscr), 
                                   LeaveWindowMask | PointerMotionMask);
 
@@ -130,6 +124,16 @@ myScreenInit (DisplayInfo *display_info, GdkScreen *gscr, unsigned long event_ma
                                   1, gdk_screen_get_height (gscr), 
                                   LeaveWindowMask | PointerMotionMask);
 
+    xfwmWindowTemp (display_info->dpy, screen_info->xroot, &screen_info->sidewalk[2], 
+                                  0, 0, 
+                                  gdk_screen_get_width (gscr), 1,
+                                  LeaveWindowMask | PointerMotionMask);
+
+    xfwmWindowTemp (display_info->dpy, screen_info->xroot, &screen_info->sidewalk[3], 
+                                  0, gdk_screen_get_height (gscr) - 1, 
+                                  gdk_screen_get_width (gscr), 1,
+                                  LeaveWindowMask | PointerMotionMask);
+
     screen_info->gnome_win = GDK_WINDOW_XWINDOW (screen_info->gtk_win->window);
 
     screen_info->net_system_tray_selection = 
@@ -171,6 +175,7 @@ myScreenInit (DisplayInfo *display_info, GdkScreen *gscr, unsigned long event_ma
     compositorManageScreen (screen_info);
 
     XDefineCursor (display_info->dpy, screen_info->xroot, myDisplayGetCursorRoot(display_info));
+
     return (screen_info);
 }
 
@@ -183,8 +188,8 @@ myScreenClose (ScreenInfo *screen_info)
     
     display_info = screen_info->display_info;
     
-    compositorUnmanageScreen (screen_info);
     clientUnframeAll (screen_info);
+    compositorUnmanageScreen (screen_info);
     closeSettings (screen_info);
 
     g_free (screen_info->workspace_names);
@@ -193,6 +198,8 @@ myScreenClose (ScreenInfo *screen_info)
 
     xfwmWindowDelete (&screen_info->sidewalk[0]);
     xfwmWindowDelete (&screen_info->sidewalk[1]);
+    xfwmWindowDelete (&screen_info->sidewalk[2]);
+    xfwmWindowDelete (&screen_info->sidewalk[3]);
     XSetInputFocus (display_info->dpy, screen_info->xroot, RevertToPointerRoot, GDK_CURRENT_TIME);
 
     g_free (screen_info->params);
